import { Component, OnInit, AfterContentInit, OnDestroy, ViewChild, ElementRef, Input, ViewChildren } from '@angular/core';
import * as BpmnJS from 'bpmn-js/dist/bpmn-viewer.production.min.js';
import { BPMNFlow } from 'src/app/models/bpmn-flow';

@Component({
  selector: 'app-bpmn-diagram',
  templateUrl: './bpmn-diagram.component.html',
  styleUrls: ['./bpmn-diagram.component.css']
})
export class BpmnDiagramComponent implements AfterContentInit, OnInit, OnDestroy {

  @Input('diagram') diagram: string;

  private bpmnJS: BpmnJS;
  private _flowToHighlight: BPMNFlow;
  private _elementToHighlight: Array<string>;

  @ViewChild('ref', { static: true }) private el: ElementRef;

  constructor() {
    this.bpmnJS = new BpmnJS();

    this.bpmnJS.on('import.done', ({ error }) => {
      if (!error) {
        this.bpmnJS.get('canvas').zoom('fit-viewport');
        this.highlight();
      }
    });
  }

  @Input('flow')
  set flowToHighlight(val: BPMNFlow) {
    this.unhighlight();
    this._flowToHighlight = val;
    this.fitViewPort();
    this.highlight();
  }
  
  @Input('elements')
  set elementToHighlight(val: Array<string>) {
    this.unhighlight();
    this._elementToHighlight = val;
    this.fitViewPort();
    this.highlight();
  }

  async ngOnInit() {
    await this.importDiagram();
  }

  ngAfterContentInit(): void {
    this.bpmnJS.attachTo(this.el.nativeElement);
  }

  ngOnDestroy(): void {
    this.bpmnJS.destroy();
  }

  importDiagram() {
    if (this.diagram) {
      this.bpmnJS.importXML(this.diagram);
    }
  }

  highlight() {
    var canvas = this.bpmnJS.get('canvas');
    
    if (this._flowToHighlight) {
      this._flowToHighlight.flowElements.forEach((element) => {
        if (element && element.id) {
          canvas.addMarker(element.id, 'testgen-highlight');
        }
      });
    }

    if(this._elementToHighlight) {
      this._elementToHighlight.forEach(element => canvas.addMarker(element, 'testgen-highlight'));
    }
  }

  unhighlight() {
    var canvas = this.bpmnJS.get('canvas');
    
    if (this._flowToHighlight) {
      this._flowToHighlight.flowElements.forEach((element) => canvas.removeMarker(element.id, 'testgen-highlight'))
    }
    
    if(this._elementToHighlight) {
      this._elementToHighlight.forEach(element => canvas.removeMarker(element, 'testgen-highlight'));
    }
  }

  fitViewPort() {
    this.bpmnJS.get('canvas').zoom('fit-viewport');
  }
}
