import { Injectable } from "@angular/core";

import { HttpInterceptor, HttpRequest, HttpHandler, HttpEvent, HttpErrorResponse } from "@angular/common/http";

import { Observable, of } from "rxjs";
import { map, catchError } from 'rxjs/operators';
import { Router } from "@angular/router";

@Injectable()
export class TokenInterceptor implements HttpInterceptor {
  constructor(public router: Router) { }
  intercept(request: HttpRequest<any>, next: HttpHandler): Observable<HttpEvent<any>> {

    console.log(request.url);

    return next.handle(request).pipe(
      map((resp: any) => {
        return resp;
      }), catchError((error, caught) => {
        this.handleError(error);
        return of(error);
      }) as any);
  }

  /**
   * Handles the Http-Status-Code-Errors 
   * If you don't want to rethrow the error, use return of(err.message)
   * If you want downstream consumers to have to handle the error as well: rethrow it
   * @param err 
   */
  private handleError(err: HttpErrorResponse): Observable<any> {
    if (err.error instanceof ProgressEvent) {
      this.router.navigateByUrl("/serverdown");
    } else
      if (err.status === 404) {
        this.router.navigateByUrl("/notfound", { replaceUrl: true });

        return of(err.message);
      }

    throw err;
  }
}