import { Injectable } from '@angular/core';
import { HttpClient } from '@angular/common/http';
import { environment } from 'src/environments/environment';
import { BPMNFlowSet } from '../models/bpmn-flow-set';
import { BPMNFlow } from '../models/bpmn-flow';
import { BPMNFlowElement } from '../models/bpmn-flow-element';
import { ExternalTopic } from '../models/external-topic';
import { Loop } from '../models/loop';

@Injectable({
  providedIn: 'root'
})
export class FlowService {

  constructor(public http: HttpClient) { }

  public async getFlow(projectId: number, flowId: string): Promise<BPMNFlow> {
    const url = `${environment.apiBaseUrl}project/${projectId}/flow/${flowId}`;
    return <BPMNFlow>await this.http.get(url).toPromise();
  }

  public async getFlows(projectId: number): Promise<BPMNFlowSet> {
    const url = `${environment.apiBaseUrl}project/${projectId}/flow`;
    return <BPMNFlowSet>await this.http.get(url).toPromise();
  }

  public async getExternalTopics(projectId: number): Promise<Array<ExternalTopic>> {
    const url = `${environment.apiBaseUrl}project/${projectId}/topics/external`;
    return <Array<ExternalTopic>>await this.http.get(url).toPromise();
  }

  public async getDelegateServiceTasks(projectId: number): Promise<Array<BPMNFlowElement>> {
    const url = `${environment.apiBaseUrl}project/${projectId}/servicetask/delegate`;
    return <Array<BPMNFlowElement>>await this.http.get(url).toPromise();
  }

  public async getLoop(projectId: number, loopId: string): Promise<Loop> {
    const url = `${environment.apiBaseUrl}project/${projectId}/loop/${loopId}`;
    return <Loop>await this.http.get(url).toPromise();
  }

  public async updateLoop(projectId: number, loop: Loop): Promise<any> {
    const url = `${environment.apiBaseUrl}project/${projectId}/loop`;
    return await this.http.put<any>(url, loop).toPromise();
  }
}
