import { Injectable } from '@angular/core';
import { HttpClient } from '@angular/common/http';
import { environment } from 'src/environments/environment';
import { BPMNTestcase } from '../models/bpmn-testcase';
import { FlowInfo } from '../models/flow-info';

@Injectable({
  providedIn: 'root'
})
export class TestService {

  constructor(public http: HttpClient) { }

  public async getTests(projectId: number, flowId: string): Promise<Array<BPMNTestcase>> {
    const url = `${environment.apiBaseUrl}project/${projectId}/flow/${flowId}/test`;
    return <Array<BPMNTestcase>>await this.http.get(url).toPromise();
  }

  public async getTest(projectId: number, testId: string): Promise<BPMNTestcase> {
    const url = `${environment.apiBaseUrl}project/${projectId}/test/${testId}`;
    return <BPMNTestcase>await this.http.get(url).toPromise();
  }

  public async getInfo(projectId: number, flowId: string): Promise<FlowInfo> {
    const url = `${environment.apiBaseUrl}project/${projectId}/flow/${flowId}/info`;
    return <FlowInfo>await this.http.get(url).toPromise(); 
  }

  public async createTest(projectId: number, test: BPMNTestcase): Promise<any> {
    const url = `${environment.apiBaseUrl}project/${projectId}/test`;
    return await this.http.post<any>(url, test).toPromise();
  }

  public async updateTest(projectId: number, testId: string, test: BPMNTestcase): Promise<any> {
    const url = `${environment.apiBaseUrl}project/${projectId}/test/${testId}`;
    return await this.http.put<any>(url, test).toPromise();
  }

  public async deleteTest(projectId: number, testId: string) {
    const url = `${environment.apiBaseUrl}project/${projectId}/test/${testId}`;
    return await this.http.delete(url).toPromise();
  }
}
